----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 11/21/2017 03:24:38 PM
-- Design Name: 
-- Module Name: seg_display - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity seg_display is
    Port ( Pitch : in STD_LOGIC_VECTOR (3 downto 0) := "0000"; -- binary number for each pitch, used in play mode
           Freq : in STD_LOGIC_VECTOR (3 downto 0); -- freq corresponding to switch positions, used in selection mode
           P_state : in STD_LOGIC_VECTOR (1 downto 0); -- output depends on state
           Clk_240Hz : in STD_LOGIC; -- switching between anodes
           Cathode : out STD_LOGIC_VECTOR (6 downto 0) := "0000000"; -- controls segments
           Anode : out STD_LOGIC_VECTOR (3 downto 0)); -- controls digits
end seg_display;

architecture Behavioral of seg_display is

    signal inc : integer := 1; -- increment between anodes/cathodes

    signal Anode1 : STD_LOGIC_VECTOR (3 downto 0) := "0111";
    
    signal Anode2 : STD_LOGIC_VECTOR (3 downto 0) := "1011";

--  andoes held constant because always using same 2

    signal Cathode1, Cathode2 : STD_LOGIC_VECTOR (6 downto 0);
    
--  2 different anode/cathode for when note needs two digits

begin

    BinDec : process (P_state, Pitch, Freq) -- decode to proper anodes/cathodes
    begin
        case (P_state) is
            when "00" =>
                case (Freq) is -- Decodes Freq switches in selection state
                    when "0001" => -- C
                        Cathode1 <= "0110001";
                        Cathode2 <= "1111111";
                    when "0010" => -- Db
                        Cathode1 <= "1000010";
                        Cathode2 <= "1100000";
                    when "0011" => -- D
                        Cathode1 <= "1000010";
                        Cathode2 <= "1111111";
                    when "0100" => -- Eb
                        Cathode1 <= "0110000";
                        Cathode2 <= "1100000";
                    when "0101" => -- E
                        Cathode1 <= "0110000";
                        Cathode2 <= "1111111";
                    when "0110" => -- F
                        Cathode1 <= "0111000";
                        Cathode2 <= "1111111";
                    when "0111" => -- Gb
                        Cathode1 <= "0000100";
                        Cathode2 <= "1100000";
                    when "1000" => -- G
                        Cathode1 <= "0000100";
                        Cathode2 <= "1111111";
                    when "1001" => -- Ab
                        Cathode1 <= "0001000";
                        Cathode2 <= "1100000";
                    when "1010" => -- A
                        Cathode1 <= "0001000";
                        Cathode2 <= "1111111";
                    when "1011" => -- Bb
                        Cathode1 <= "0000000";
                        Cathode2 <= "1100000";
                    when "1100" => -- B
                        Cathode1 <= "0000000";
                        Cathode2 <= "1111111";
                    when others => -- 0 for a rest
                        Cathode1 <= "0000001";
                        Cathode2 <= "1111111";
                end case;
            when "01" => -- decodes Pitch in play state
                case (Pitch) is
                    when "0001" =>
                        Cathode1 <= "0110001";
                        Cathode2 <= "1111111";
                    when "0010" =>
                        Cathode1 <= "1000010";
                        Cathode2 <= "1100000";
                    when "0011" =>
                        Cathode1 <= "1000010";
                        Cathode2 <= "1111111";
                    when "0100" =>
                        Cathode1 <= "0110000";
                        Cathode2 <= "1100000";
                    when "0101" =>
                        Cathode1 <= "0110000";
                        Cathode2 <= "1111111";
                    when "0110" =>
                        Cathode1 <= "0111000";
                        Cathode2 <= "1111111";
                    when "0111" =>
                        Cathode1 <= "0000100";
                        Cathode2 <= "1100000";
                    when "1000" =>
                        Cathode1 <= "0000100";
                        Cathode2 <= "1111111";
                    when "1001" =>
                        Cathode1 <= "0001000";
                        Cathode2 <= "1100000";
                    when "1010" =>
                        Cathode1 <= "0001000";
                        Cathode2 <= "1111111";
                    when "1011" =>
                        Cathode1 <= "0000000";
                        Cathode2 <= "1100000";
                    when "1100" =>
                        Cathode1 <= "0000000";
                        Cathode2 <= "1111111";
                    when others =>
                        Cathode1 <= "0000001";
                        Cathode2 <= "1111111";
                end case;
            when "10" =>
                Cathode1 <= "1101011"; -- pause symbol
                Cathode2 <= "1111111";
            when others => -- catch, should never occur
                Cathode1 <= "1111111";
                Cathode2 <= "1111111";
        end case;
        
    end process;
    
    switch : process (Clk_240Hz) -- switch between digits
    begin
        if (rising_edge(Clk_240Hz)) then
            inc <= inc + 1; -- ambigous counter
            case (inc) is
                when 0 =>
                    Anode <= Anode1;
                    Cathode <= Cathode1;
                when others =>
                    inc <= 0;
                    Anode <= Anode2;
                    Cathode <= Cathode2;
            end case;
        end if;
    end process;
    
end Behavioral;