----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date: 11/25/2017 11:29:19 AM
-- Design Name: 
-- Module Name: top_block - Behavioral
-- Project Name: 
-- Target Devices: 
-- Tool Versions: 
-- Description: 
-- 
-- Dependencies: 
-- 
-- Revision:
-- Revision 0.01 - File Created
-- Additional Comments:
-- 
----------------------------------------------------------------------------------


library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx leaf cells in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity top_block is
    Port ( Clk : in STD_LOGIC; 
           PP : in STD_LOGIC; -- play/ pause
           Sel : in STD_LOGIC; -- selection mode
           Assign : in STD_LOGIC; -- assign a note
           Freq : in STD_LOGIC_VECTOR (3 downto 0); -- frequency switches
           Step : in STD_LOGIC_VECTOR (3 downto 0); -- step switches
           DAC : out STD_LOGIC_VECTOR (3 downto 0); -- output to dac
           Anode : out STD_LOGIC_VECTOR (3 downto 0); 
           Cathode : out STD_LOGIC_VECTOR (6 downto 0));
end top_block;

architecture Behavioral of top_block is

    component FSM is
        Port ( Clk : in STD_LOGIC;
               Sel : in STD_LOGIC; -- put in selection mode
               PP : in STD_LOGIC;  -- play/ pause
               P_state : out STD_LOGIC_VECTOR (1 downto 0)); -- PS output for output logic
    end component;
    
    component Clk_div_7seg is
        Port ( Clk : in STD_LOGIC;
               Clk_7seg : out STD_LOGIC);
    end component;
    
    component Clk_div_BPM is
        Port ( Clk : in STD_LOGIC;
               Clk_BPM : out STD_LOGIC);
    end component;
    
    component Clk_div_pitches is
        Port ( Clk : in STD_LOGIC;
               Clk_C : out STD_LOGIC;
               Clk_Db : out STD_LOGIC;
               Clk_D : out STD_LOGIC;
               Clk_Eb : out STD_LOGIC;
               Clk_E : out STD_LOGIC;
               Clk_F : out STD_LOGIC;
               Clk_Gb : out STD_LOGIC;
               Clk_G : out STD_LOGIC;
               Clk_Ab : out STD_LOGIC;
               Clk_A : out STD_LOGIC;
               Clk_Bb : out STD_LOGIC;
               Clk_B : out STD_LOGIC);
    end component;
    
    component Note_assign is
        Port ( P_state : in STD_LOGIC_VECTOR (1 downto 0);
               Sel : in STD_LOGIC;
               Switch : in STD_LOGIC; -- positional switches
               Freq : in STD_LOGIC_VECTOR (3 downto 0); -- the combination of freq switches
               Assign : in STD_LOGIC;
               Pitch : out STD_LOGIC_VECTOR (3 downto 0) := "0000"); -- the assigned pitch 
    end component;
    
    component seg_display is
        Port ( Pitch : in STD_LOGIC_VECTOR (3 downto 0); -- binary number for each pitch, used in play mode
               Freq : in STD_LOGIC_VECTOR (3 downto 0); -- freq corresponding to switch positions, used in selection mode
               P_state : in STD_LOGIC_VECTOR (1 downto 0); -- output depends on state
               Clk_240Hz : in STD_LOGIC; -- switching between anodes
               Cathode : out STD_LOGIC_VECTOR (6 downto 0); 
               Anode : out STD_LOGIC_VECTOR (3 downto 0));
    end component;
    
    component output_select is
        Port ( Pitch : in STD_LOGIC_VECTOR (3 downto 0);
               C : in STD_LOGIC;
               Db : in STD_LOGIC;
               D : in STD_LOGIC;
               Eb : in STD_LOGIC;
               E : in STD_LOGIC;
               F : in STD_LOGIC;
               Gb : in STD_LOGIC;
               G : in STD_LOGIC;
               Ab : in STD_LOGIC;
               A : in STD_LOGIC;
               Bb : in STD_LOGIC;
               B : in STD_LOGIC;
               Tone : out STD_LOGIC); -- tone for each note
    end component;
    
    component square_wave is
        Port ( Tone : in STD_LOGIC;
               DAC_input : out STD_LOGIC_VECTOR (3 downto 0));
    end component;
    
    component fin_sel is
        Port ( PS : in STD_LOGIC_VECTOR (1 downto 0);
               DAC_input : in STD_LOGIC_VECTOR (3 downto 0);
               DAC : out STD_LOGIC_VECTOR (3 downto 0));
    end component;
    
    component output_FSM is
        Port ( Clk_BPM : in STD_LOGIC;
               FSM1_state : in STD_LOGIC_VECTOR (1 downto 0);
               Pitch_in : in STD_LOGIC_VECTOR (15 downto 0);
               Pitch_out : out STD_LOGIC_VECTOR (3 downto 0) := "0000");
    end component;

    signal state : STD_LOGIC_VECTOR (1 downto 0); -- PS internal signal
    
    signal seg7, BPM : STD_LOGIC; -- clock signals
    
    signal C, Db, D, Eb, E, F, Gb, G, Ab, A, Bb, B : STD_LOGIC; -- clock signals
    
    signal pitches : STD_LOGIC_VECTOR (15 downto 0) := x"0000"; -- pitches from note assign to ouput module
    
    signal last_pitch : STD_LOGIC_VECTOR (3 downto 0) := "0000"; -- output_fsm to output_select
    
    signal tone_fin : STD_LOGIC := '0'; -- fsm2 selected pitch frequency
    
    signal dac_input : STD_LOGIC_VECTOR (3 downto 0) := "0000"; -- connect sgwv and fin_sel

begin

    -- play/ pause/ select finite state machine
    FSM1 : FSM port map (Clk => Clk, Sel => Sel, PP => PP, P_state => state);
    
    -- clk signal for siwtching between 7_seg digits
    Clk_7seg : Clk_div_7seg port map (Clk => Clk, Clk_7seg => seg7);
    
    -- clk signal for switching between notes to the board
    Clk_BPM : Clk_div_BPM port map (Clk => Clk, Clk_BPM => BPM);
    
    -- clk signals for pitch frequencies
    Clk_pitches : Clk_div_pitches port map (Clk => Clk, Clk_C => C, Clk_Db => Db,
                    Clk_D => D, Clk_Eb => Eb, Clk_E => E, Clk_F => F, Clk_Gb => Gb,
                    Clk_G => G, Clk_Ab => Ab, Clk_A => A, Clk_Bb => Bb, Clk_B => B);
    
    -- assigning each step a pitch
    NA3 : note_assign port map (P_state => state, Sel => Sel, Switch => Step(3), Freq => Freq, 
            Assign => Assign, Pitch => pitches(15 downto 12));

    NA2 : note_assign port map (P_state => state, Sel => Sel, Switch => Step(2), Freq => Freq, 
            Assign => Assign, Pitch => pitches(11 downto 8));
            
    NA1 : note_assign port map (P_state => state, Sel => Sel, Switch => Step(1), Freq => Freq, 
            Assign => Assign, Pitch => pitches(7 downto 4));

    NA0 : note_assign port map (P_state => state, Sel => Sel, Switch => Step(0), Freq => Freq, 
            Assign => Assign, Pitch => pitches(3 downto 0));
     
    -- fsm for switching between notes        
    OFSM : output_FSM port map (Clk_BPM => BPM, FSM1_state => state, Pitch_in => pitches, Pitch_out => last_pitch);
    
    -- assigning the output a clk frequency
    OS : output_select port map (Pitch => last_pitch, C => C, Db => Db, D => D, Eb => Eb,
            E => E, F => F, Gb => Gb, G => G, Ab => Ab, A => A, Bb => Bb, B =>B, Tone => tone_fin);
           
    -- 7seg display
    SSEG : seg_display port map (Pitch => last_pitch, Freq => Freq, P_state => state, 
            Clk_240Hz => seg7, Cathode => Cathode, Anode => Anode); -- seg7
     
    -- creating the square wave for each selected frequency
    SQWV : square_wave port map (Tone => Tone_fin, DAC_input => dac_input);
    
    
    -- selecting the output based off the present state 
    FSel : fin_sel port map (PS => state, DAC_input => dac_input, DAC => DAC);  
              
end Behavioral;
